@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef AGENT_H
@@ -122,7 +122,7 @@ struct
 
   /* If set, a passphrase history will be written and checked at each
      passphrase change.  */
-  int enable_passhrase_history;
+  int enable_passphrase_history;
 
   int running_detached; /* We are running detached from the tty. */
 
@@ -154,12 +154,12 @@ struct
   int keep_tty;      /* Don't switch the TTY (for pinentry) on request */
   int keep_display;  /* Don't switch the DISPLAY (for pinentry) on request */
 
-  /* This global options indicates the use of an extra socket. Note
+  /* This global option indicates the use of an extra socket. Note
      that we use a hack for cleanup handling in gpg-agent.c: If the
      value is less than 2 the name has not yet been malloced. */
   int extra_socket;
 
-  /* This global options indicates the use of an extra socket for web
+  /* This global option indicates the use of an extra socket for web
      browsers. Note that we use a hack for cleanup handling in
      gpg-agent.c: If the value is less than 2 the name has not yet
      been malloced. */
@@ -349,7 +349,8 @@ void agent_sighup_action (void);
 int map_pk_openpgp_to_gcry (int openpgp_algo);
 
 /*-- command.c --*/
-gpg_error_t agent_inq_pinentry_launched (ctrl_t ctrl, unsigned long pid);
+gpg_error_t agent_inq_pinentry_launched (ctrl_t ctrl, unsigned long pid,
+                                         const char *extra);
 gpg_error_t agent_write_status (ctrl_t ctrl, const char *keyword, ...)
      GPGRT_ATTR_SENTINEL(0);
 gpg_error_t agent_print_status (ctrl_t ctrl, const char *keyword,
@@ -490,6 +491,9 @@ gpg_error_t s2k_hash_passphrase (const char *passphrase, int hashalgo,
                                  const unsigned char *s2ksalt,
                                  unsigned int s2kcount,
                                  unsigned char *key, size_t keylen);
+gpg_error_t agent_write_shadow_key (const unsigned char *grip,
+                                    const char *serialno, const char *keyid,
+                                    const unsigned char *pkbuf, int force);
 
 
 /*-- trustlist.c --*/
