@@ -25,7 +25,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -195,6 +195,21 @@ map_eai_to_gpg_error (int ec)
   return err;
 }
 
+#ifdef USE_ADNS
+static gpg_error_t
+map_adns_status_to_gpg_error (adns_status status)
+{
+  gpg_err_code_t ec;
+
+  switch (status)
+    {
+    /* case adns_s_netunreach: ec = GPG_ERR_ENETUNREACH; break; */
+    default: ec = GPG_ERR_GENERAL; break;
+    }
+  return gpg_error (ec);
+}
+#endif /*USE_ADNS*/
+
 
 #ifdef USE_ADNS
 /* Init ADNS and store the new state at R_STATE.  Returns 0 on
@@ -278,7 +293,7 @@ resolve_name_adns (const char *name, unsigned short port,
   my_protect ();
   if (ret)
     {
-      err = gpg_error_from_syserror ();
+      err = gpg_error (gpg_err_code_from_errno (ret));
       log_error ("DNS query failed: %s\n", gpg_strerror (err));
       goto leave;
     }
@@ -286,6 +301,9 @@ resolve_name_adns (const char *name, unsigned short port,
   err = gpg_error (GPG_ERR_NOT_FOUND);
   if (answer->status != adns_s_ok || answer->type != adns_r_addr)
     {
+      err = map_adns_status_to_gpg_error (answer->status);
+      if (gpg_err_code (err) == GPG_ERR_GENERAL)
+        err = gpg_error (GPG_ERR_NOT_FOUND);
       log_error ("DNS query returned an error: %s (%s)\n",
                  adns_strerror (answer->status),
                  adns_errabbrev (answer->status));
@@ -682,8 +700,8 @@ get_dns_cert (const char *name, int want_certtype,
   my_protect ();
   if (ret)
     {
-      err = gpg_error_from_syserror ();
-      /* log_error ("DNS query failed: %s\n", strerror (errno)); */
+      err = gpg_error (gpg_err_code_from_errno (ret));
+      /* log_error ("DNS query failed: %s\n", gpg_strerror (err)); */
       adns_finish (state);
       return err;
     }
@@ -692,7 +710,9 @@ get_dns_cert (const char *name, int want_certtype,
       /* log_error ("DNS query returned an error: %s (%s)\n", */
       /*            adns_strerror (answer->status), */
       /*            adns_errabbrev (answer->status)); */
-      err = gpg_error (GPG_ERR_NOT_FOUND);
+      err = map_adns_status_to_gpg_error (answer->status);
+      if (gpg_err_code (err) == GPG_ERR_GENERAL)
+        err = gpg_error (GPG_ERR_NOT_FOUND);
       goto leave;
     }
 
@@ -990,7 +1010,7 @@ get_dns_cert (const char *name, int want_certtype,
   if (r_key)
     *r_key = NULL;
   if (r_keylen)
-    *r_keylen = NULL;
+    *r_keylen = 0;
   *r_fpr = NULL;
   *r_fprlen = 0;
   *r_url = NULL;
@@ -1036,7 +1056,7 @@ getsrv (const char *name,struct srventry **list)
     my_protect ();
     if (rc)
       {
-        log_error ("DNS query failed: %s\n", strerror (errno));
+        log_error ("DNS query failed: %s\n", strerror (rc));
         adns_finish (state);
         return -1;
       }
@@ -1095,7 +1115,9 @@ getsrv (const char *name,struct srventry **list)
     if (tor_mode)
       return -1;
 
+    my_unprotect ();
     r = res_query (name, C_IN, T_SRV, answer, sizeof answer);
+    my_protect ();
     if (r < sizeof (HEADER) || r > sizeof answer
         || header->rcode != NOERROR || !(count=ntohs (header->ancount)))
       return 0; /* Error or no record found.  */
@@ -1281,7 +1303,7 @@ get_dns_cname (const char *name, char **r_cname)
     my_protect ();
     if (rc)
       {
-        err = gpg_error_from_syserror ();
+        err = gpg_error (gpg_err_code_from_errno (rc));
         log_error ("DNS query failed: %s\n", gpg_strerror (err));
         adns_finish (state);
         return err;
@@ -1289,7 +1311,7 @@ get_dns_cname (const char *name, char **r_cname)
     if (answer->status != adns_s_ok
         || answer->type != adns_r_cname || answer->nrrs != 1)
       {
-        err = gpg_error (GPG_ERR_GENERAL);
+        err = map_adns_status_to_gpg_error (answer->status);
         log_error ("DNS query returned an error or no records: %s (%s)\n",
                    adns_strerror (answer->status),
                    adns_errabbrev (answer->status));
