@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -32,6 +32,7 @@
 
 #include "private.h"
 #include "scheme.h"
+#include "scheme-private.h"
 #include "ffi.h"
 #include "i18n.h"
 #include "../../common/argparse.h"
@@ -176,6 +177,13 @@ load (scheme *sc, char *file_name,
   scheme_load_named_file (sc, h, qualified_name);
   fclose (h);
 
+  if (sc->retcode)
+    {
+      if (sc->nesting)
+        fprintf (stderr, "%s: Unbalanced parenthesis\n", qualified_name);
+      return gpg_error (GPG_ERR_GENERAL);
+    }
+
   if (file_name != qualified_name)
     free (qualified_name);
   return 0;
@@ -222,7 +230,7 @@ main (int argc, char **argv)
   i18n_init ();
   init_common_subsystems (&argc, &argv);
 
-  if (!gcry_check_version (GCRYPT_VERSION))
+  if (!gcry_check_version (NEED_LIBGCRYPT_VERSION))
     {
       fputs ("libgcrypt version mismatch\n", stderr);
       exit (2);
@@ -255,7 +263,8 @@ main (int argc, char **argv)
   if (! err)
     err = load (sc, "ffi.scm", 0, 1);
   if (! err)
-    err = ffi_init (sc, argv0, argc, (const char **) argv);
+    err = ffi_init (sc, argv0, script ? script : "interactive",
+                    argc, (const char **) argv);
   if (! err)
     err = load (sc, "lib.scm", 0, 1);
   if (! err)
@@ -282,6 +291,7 @@ main (int argc, char **argv)
         log_fatal ("%s: %s", script, gpg_strerror (err));
     }
 
+  scheme_load_string (sc, "(*run-atexit-handlers*)");
   scheme_deinit (sc);
   xfree (sc);
   return EXIT_SUCCESS;
